% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/species_params.R
\name{species_params}
\alias{species_params}
\alias{species_params<-}
\title{Species parameters}
\usage{
species_params(params)

species_params(params) <- value
}
\arguments{
\item{params}{A MizerParams object}

\item{value}{A data frame with the species parameters}
}
\description{
These functions allow you to get or set the species parameters stored in
a MizerParams object.
}
\details{
The \code{species_params} data frame holds species-specific parameters. The data
frame has one row for each species and one column for each species parameter.
There are a lot of species parameters and we will list them all below, but
most of them have sensible default values. The only required columns are
\code{species} for the species name and \code{w_inf} for its asymptotic size. However
if you have information about the values of other parameters then you should
include them in the \code{species_params} data frame.
\subsection{Species parameters for setting up size-dependent parameters}{

There are some species parameters that are used to set up the
size-dependent parameters that are used in the mizer model:
\itemize{
\item \code{gamma} and \code{q} are used to set the search volume, see \code{\link[=setSearchVolume]{setSearchVolume()}}.
\item \code{h} and \code{n} are used to set the maximum intake rate, see \code{\link[=setMaxIntakeRate]{setMaxIntakeRate()}}.
\item \code{k}, \code{ks} and \code{p} are used to set activity and basic metabolic rate,
see \code{\link[=setMetabolicRate]{setMetabolicRate()}}.
\item \code{z0} is used to set the external mortality rate, see \code{\link[=setExtMort]{setExtMort()}}.
\item \code{w_mat}, \code{w_mat25}, \code{w_inf} and \code{m} are used to set the allocation to
reproduction, see \code{\link[=setReproduction]{setReproduction()}}.
\item \code{pred_kernel_type} specifies the shape of the predation kernel. The default
is a "lognormal", for other options see the "Setting predation kernel"
section in the help for \code{\link[=setPredKernel]{setPredKernel()}}.
\item \code{beta} and \code{sigma} are parameters of the lognormal predation kernel, see
\code{\link[=lognormal_pred_kernel]{lognormal_pred_kernel()}}. There will be other parameters if you are
using other predation kernel functions.
}

When you change one of the above species parameters in an already existing
MizerParams object using \code{\link[=species_params<-]{species_params<-()}}, the new value will be
used to update the corresponding size-dependent rates automatically, unless
you have set those size-dependent rates manually, in which case the
corresponding species parameters will be ignored.
}

\subsection{Species parameters used in model directly}{

There are some species parameters that are used directly in the model
rather than being used for setting up size-dependent parameters:
\itemize{
\item \code{alpha} is the assimilation efficiency, the proportion of the consumed
biomass that can be used for growth, metabolism and reproduction, see
the help for \code{\link[=getEReproAndGrowth]{getEReproAndGrowth()}}.
\item \code{w_min} is the egg size.
\item \code{interaction_resource} sets the interaction strength with the resource,
see "Predation encounter" section in the help for \code{\link[=getEncounter]{getEncounter()}}.
\item \code{erepro} is the reproductive efficiency, the proportion of the energy
invested into reproduction that is converted to egg biomass, see
\code{\link[=getRDI]{getRDI()}}.
\item \code{Rmax} is the parameter in the Beverton-Holt density dependence added to
the reproduction, see \code{\link[=setBevertonHolt]{setBevertonHolt()}}. There will be other such
parameters if you use other density dependence functions, see the
"Density dependence" section in the help for \code{\link[=setReproduction]{setReproduction()}}.
}

Two parameters are used only by functions that need to convert between
weight and length:
\itemize{
\item \code{a} and \code{b} are the parameters in the allometric weight-length
relationship \eqn{w = a l ^ b}.
}
}

\subsection{Species parameters to calculate defaults for others}{

Not all of species parameters have to be specified by the user. If they are
missing, \code{\link[=newMultispeciesParams]{newMultispeciesParams()}} will give them default values, sometimes
by using other species parameters. The parameters that are only used to
calculate default values for other parameters are:
\itemize{
\item \code{k_vb} and \code{t0} are the von Bertalanffy growth parameters and are used
together with the length-weight relationship exponent \code{b} and the egg
size \code{w_min} to
get a default value for the coefficient of the maximum intake rate \code{h},
see \code{\link[=get_h_default]{get_h_default()}}.
\item \code{f0} is the feeding level and is used to get a default value for the
coefficient of the search volume \code{gamma}, see \code{\link[=get_gamma_default]{get_gamma_default()}}.
\item \code{fc} is the critical feeding level below which the species can not
maintain itself. This is used to get a default value for the coefficient
of the metabolic rate \code{ks}, see \code{\link[=get_ks_default]{get_ks_default()}}.
}

Note that these parameters
will only be used when setting up a new model with
\code{\link[=newMultispeciesParams]{newMultispeciesParams()}}. Changing them later will have no effect
because the default for the other parameters will not be recalculated.
}

\subsection{Species parameters containing observations}{

There are other species parameters that are used in tuning the model to
observations:
\itemize{
\item \code{biomass_observed} and \code{biomass_cutoff} allow you to specify for each
species the total observed biomass above some cutoff size. This is
used by \code{\link[=calibrateBiomass]{calibrateBiomass()}} and \code{\link[=matchBiomasses]{matchBiomasses()}}.
\item \code{yield_observed} allows you to specify for each
species the total annual fisheries yield. This is
used by \code{\link[=calibrateYield]{calibrateYield()}} and \code{\link[=matchYields]{matchYields()}}.
}
}

\subsection{Species parameters influencing plots}{

Finally there are two species parameters that control the way the species are
represented in plots:
\itemize{
\item \code{linecolour} specifies the colour and can be any valid R colour value.
\item \code{linetype} specifies the line type ("solid", "dashed", "dotted", "dotdash",
"longdash", "twodash" or "blank")
}
}

\subsection{Species parameters related to fishing gears}{

Other species-specific information that is related to how the species is
fished is specified in a gear parameter data frame, see \code{\link[=gear_params]{gear_params()}}.
However in the case where each species is caught by only a single gear,
this information can also optionally be provided as columns in the
\code{species_params} data frame and \code{\link[=newMultispeciesParams]{newMultispeciesParams()}} will transfer
them to the \code{gear_params} data frame. However changing these parameters later
in the species parameter data frame will have no effect.
}

\subsection{Your own species parameters}{

You are allowed to include additional columns in the \code{species_params}
data frame. They will simply be ignored by mizer but will be stored in the
MizerParams object, in case your own code makes use of them.
}
}
\seealso{
\code{\link[=validSpeciesParams]{validSpeciesParams()}}

Other functions for setting parameters: 
\code{\link{gear_params}()},
\code{\link{resource_params}()},
\code{\link{setExtMort}()},
\code{\link{setFishing}()},
\code{\link{setInitialValues}()},
\code{\link{setInteraction}()},
\code{\link{setMaxIntakeRate}()},
\code{\link{setMetabolicRate}()},
\code{\link{setParams}()},
\code{\link{setPredKernel}()},
\code{\link{setReproduction}()},
\code{\link{setResource}()},
\code{\link{setSearchVolume}()}
}
\concept{functions for setting parameters}
